#
# Utility to transcode the Zeitgeist mimetype- and URI scheme mappings
# from Python code to a C file defining a building these mappings which
# can be accessed via and API defined in zeitgeist-mimetypes.h
#

import sys
import re
import zeitgeist.datamodel as dm
from zeitgeist.mimetypes import MIMES, MIMES_REGEX, SCHEMES

def caseconv(name):
	"""
	Converts CamelCase to CAMEL_CASE
	"""
	result = ""
	for i in range(len(name) - 1) :
		if name[i].islower() and name[i+1].isupper():
			result += name[i].upper() + "_"
		else:
			result += name[i].upper()
	result += name[-1].upper()
	return result

def symbolname (symbol):
	try:
		ns, name = symbol[symbol.rfind("/")+1:].split("#")
		return "ZEITGEIST_%s_%s" % (ns.upper(), caseconv(name))
	except Exception, e:
		return symbol.uri.rpartition("#")[2].upper()

def output_load_funcs (mimes_body, schemes_body):
	mimes_header = """
static void
_ensure_mimes_loaded (void)
{
  if (mimes_loaded) return;

%s

  mimes_loaded = TRUE;
}
"""

	schemes_header = """
static void
_ensure_schemes_loaded (void)
{
  if (schemes_loaded) return;

%s

  schemes_loaded = TRUE;
}
"""

	# indent
	m = mimes_header[1:] % re.sub(r'(?m)^(.+)$', r'  \1', mimes_body)
	s = schemes_header[1:] % re.sub(r'(?m)^(.+)$', r'  \1', schemes_body)
	return '\n'.join([m, s])

def get_mimetypes_calls (simple_mimetypes, regex_mimetypes):
	calls = []
	for mimetype, interpretation in simple_mimetypes.items():
		calls.append('zeitgeist_register_mimetype ("%s", %s);' % (mimetype, symbolname(interpretation)))

	for mimetype_regex, interpretation in regex_mimetypes:
		mimetype = re.sub(r'\\', '\\' * 4, str(mimetype_regex))
		calls.append('zeitgeist_register_mimetype_regex ("%s", %s);' % (mimetype, symbolname(interpretation)))

	return calls

def get_schemes_calls (schemes):
	calls = []
	for scheme, manifestation in schemes:
		calls.append('zeitgeist_register_uri_scheme ("%s", %s);' % (scheme, symbolname(manifestation)))

	return calls

if __name__ == "__main__":
	all_mimes = get_mimetypes_calls(MIMES, MIMES_REGEX)
	all_schemes = get_schemes_calls(SCHEMES)

	assert(len(all_mimes) > 0 and len(all_schemes) > 0)

	f = open('zeitgeist-mimetypes.template', 'r')
	print f.read()
	print output_load_funcs('\n'.join(all_mimes), '\n'.join(all_schemes))
